;;; highline.el --- Minor mode to highlight current line in buffer.

;; Copyright (C) 2000 Vinicius Jose Latorre

;; Author:	Vinicius Jose Latorre <vinicius@cpqd.com.br>
;; Maintainer:	Vinicius Jose Latorre <vinicius@cpqd.com.br>
;; Keywords:	faces, frames, editing
;; Time-stamp:	<2000/02/09 10:40:24 vinicius>
;; Version:	1.0

;; This file is *NOT* (yet?) part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Introduction
;; ------------
;;
;; This package is a minor mode to highlight the current line in buffer.
;;
;; highline was inspired on:
;;
;;    linemenu.el
;;		Bill Brodie <wbrodie@panix.com>
;;    hl-line.el
;;		Dave Love <fx@gnu.org>
;;    highlight-current-line.el
;;		Christoph Conrad <Christoph.Conrad@post.rwth-aachen.de>
;;
;; To use highline, insert in your ~/.emacs:
;;
;;    (require 'highline)
;;
;; Or:
;;
;;    (autoload 'highline-on         "highline"
;;      "Turn on highlightining current line in buffer."   t)
;;    (autoload 'highline-off        "highline"
;;      "Turn off highlightining current line in buffer."  t)
;;    (autoload 'highline-local-mode "highline"
;;      "Toggle local minor mode to highlight current line in buffer."  t)
;;    (autoload 'highline-mode       "highline"
;;      "Toggle global minor mode to highlight current line in buffer." t)
;;    (autoload 'highline-customize  "highline"
;;      "Customize highlightining current line in buffer." t)
;;
;; For good performance, be sure to byte-compile highline.el, e.g.
;;
;;    M-x byte-compile-file <give the path to highline.el when prompted>
;;
;; This will generate highline.elc, which will be loaded instead of
;; highline.el.
;;
;; highline was tested with GNU Emacs 20.4.1.
;;
;;
;; Using highline
;; --------------
;;
;; To activate highline locally, type:
;;
;;    M-x highline-on RET
;;
;; Or:
;;
;;    C-u 1 M-x highline-local-mode RET
;;
;; To deactivate highline locally, type:
;;
;;    M-x highline-off RET
;;
;; Or:
;;
;;    C-u 0 M-x highline-local-mode RET
;;
;; To toggle highline locally, type:
;;
;;    M-x highline-local-mode RET
;;
;; To toggle highline globally, type:
;;
;;    M-x highline-mode RET
;;
;; To customize highline, type:
;;
;;    M-x highline-customize RET
;;
;; You can also bind `highline-local-mode', `highline-mode', `highline-on',
;; `highline-off' and `highline-customize' to some key, like:
;;
;;    (global-set-key "\C-c\C-a" 'highline-on)
;;    (global-set-key "\C-c\C-e" 'highline-off)
;;    (global-set-key "\C-c\C-l" 'highline-local-mode)
;;    (global-set-key "\C-c\C-g" 'highline-mode)
;;    (global-set-key "\C-c\C-c" 'highline-customize)
;;
;; NOTE: To avoid confusion, avoid mix highline local and global mode use.
;;	 Instead, use highline only locally or only globally.
;;
;;
;; Example
;; -------
;;
;; As an example, try to insert this in your .emacs file:
;;
;;  (require 'highline)
;;  ;; Turn on local highlighting for Dired (C-x d)
;;  (add-hook 'dired-after-readin-hook 'highline-on)
;;  ;; Turn on local highlighting for list-buffers (C-x C-b)
;;  (defadvice list-buffers (after highlight-line activate)
;;    (save-excursion
;;      (set-buffer "*Buffer List*")
;;      (highline-on)))
;;
;;
;; Hooks
;; -----
;;
;; highline has the following hook variables:
;;
;; `highline-hook'
;;    It is evaluated always when highline is turned on globally.
;;
;; `highline-local-hook'
;;    It is evaluated always when highline is turned on locally.
;;
;; `highline-load-hook'
;;    It is evaluated after highline package is loaded.
;;
;;
;; Options
;; -------
;;
;; Below it's shown a brief description of highline options, please, see the
;; options declaration in the code for a long documentation.
;;
;; `highline-face'			Face used to highlight the current
;;					line.
;;
;; `highline-whole-line'		Non-nil means mark up to end-of-line;
;;					otherwise, mark up to window-border.
;;
;; `highline-verbose'			Non-nil means generate messages.
;;
;; `highline-ignore-regexp'		Regexp for buffers to ignore.
;;
;; To set the above options you may:
;;
;; a) insert the code in your ~/.emacs, like:
;;
;;	 (setq highline-face 'highlight)
;;
;;    This way always keep your default settings when you enter a new Emacs
;;    session.
;;
;; b) or use `set-variable' in your Emacs session, like:
;;
;;	 M-x set-variable RET highline-face RET highlight RET
;;
;;    This way keep your settings only during the current Emacs session.
;;
;; c) or use customization, for example:
;;	 click on menu-bar *Help* option,
;;	 then click on *Customize*,
;;	 then click on *Browse Customization Groups*,
;;	 expand *Editing* group,
;;	 expand *Highline* group
;;	 and then customize highline options.
;;    Through this way, you may choose if the settings are kept or not when
;;    you leave out the current Emacs session.
;;
;; d) or see the option value:
;;
;;	 C-h v highline-face RET
;;
;;    and click the *customize* hypertext button.
;;    Through this way, you may choose if the settings are kept or not when
;;    you leave out the current Emacs session.
;;
;; e) or invoke:
;;
;;	 M-x highline-customize RET
;;
;;    and then customize highline options.
;;    Through this way, you may choose if the settings are kept or not when
;;    you leave out the current Emacs session.
;;
;;
;; Things To Change
;; ----------------
;;
;; . At moment, nothing.  Any idea?  Send it!
;;
;;
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Code:


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; User Variables:


;;; Interface to the command system

(defgroup highline nil
  "Highlight the current line"
  :link '(emacs-library-link :tag "Source Lisp File" "highline.el")
  :group 'faces
  :group 'frames
  :group 'editing)


(defcustom highline-face 'highline-face
  "*Symbol face used to highlight the current line."
  :type 'face
  :group 'highline)


(defface highline-face '((t (:background "paleturquoise")))
  "Face used to highlight current line.")


(defcustom highline-whole-line t
  "*Non-nil means mark up to end-of-line; otherwise, mark up to window-border."
  :type 'boolean
  :group 'highline)


(defcustom highline-verbose t
  "*Non-nil means generate messages."
  :type 'boolean
  :group 'highline)


(defcustom highline-ignore-regexp
  (concat "Faces\\|Colors\\|Minibuf"
	  ;; for example:
	  ;; "\\|RMAIL.*summary\\|\\*Group\\|\\*Summary"
	  )
  "*Regexp for buffers to ignore.

Set to nil or \"\", to accept any buffer.

Used by `highline-highlight-current-line'."
  :type 'regexp
  :group 'highline)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Macro


(defmacro highline-message (&rest body)
  (` (and highline-verbose (interactive-p)
	  (message (,@ body)))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Customization


;;;###autoload
(defun highline-customize ()
  "Customize highline group."
  (interactive)
  (customize-group 'highline))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; User commands


(defvar highline-mode nil
  "Non-nil means highline global minor mode is enabled (HL on modeline).")


(defvar highline-local-mode nil
  "Non-nil means highline local minor mode is enabled (hl on modeline).")
(make-variable-buffer-local 'highline-local-mode)


(defvar highline-overlay nil
  "Highlight for current line")
(make-variable-buffer-local 'highline-overlay)


;;;###autoload
(defun highline-mode (&optional arg)
  "Toggle global minor mode to highlight the line about point.

With ARG, turn highline mode on if ARG is positive, off otherwise.
Only useful with a windowing system."
  (interactive "P")
  (setq highline-mode (if arg
			  (> (prefix-numeric-value arg) 0)
			(not highline-mode)))
  (cond
   (highline-mode			; turn on global mode
    (add-hook 'post-command-hook 'highline-highlight-current-line)
    (highline-highlight-current-line)
    (run-hooks 'highline-hook))
   (highline-local-mode			; turn off only if local mode is off
    (highline-off))
   (t					; indicate that global mode still is on
    (setq highline-mode t))
   )
  (highline-message "Highline global mode is %s" (if highline-mode "on" "off")))


;;;###autoload
(defun highline-local-mode (&optional arg)
  "Toggle local minor mode to highlight the line about point.

With ARG, turn highline mode on if ARG is positive, off otherwise.
Only useful with a windowing system."
  (interactive "P")
  (if (if arg
	  (> (prefix-numeric-value arg) 0)
	(not highline-local-mode))
      (highline-on)
    (highline-off))
  (highline-message "Highline local mode is %s"
		    (if highline-local-mode "on" "off")))


;;;###autoload
(defun highline-on ()
  "Turn on highlighting of the current line in this buffer."
  (interactive)
  (setq highline-local-mode t)
  (make-local-variable 'post-command-hook)
  (add-hook 'post-command-hook 'highline-highlight-current-line)
  (highline-highlight-current-line)
  (run-hooks 'highline-local-hook)
  (highline-message "Highline local mode is on"))


;;;###autoload
(defun highline-off ()
  "Turn off highlighting of the current line in this buffer."
  (interactive)
  (setq highline-local-mode nil)
  (and highline-overlay
       (setq highline-overlay (move-overlay highline-overlay 1 1)))
  (remove-hook 'post-command-hook 'highline-highlight-current-line t)
  (highline-message "Highline local mode is off"))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Internal function


(defun highline-highlight-current-line ()
  "Highlight current line."
  (unless (and highline-ignore-regexp
	       (not (equal "" highline-ignore-regexp))
	       (string-match highline-ignore-regexp (buffer-name)))
    (unless highline-overlay
      (setq highline-overlay (make-overlay 1 1)) ; Hide it for now
      (overlay-put highline-overlay 'hilit t)
      (overlay-put highline-overlay 'face highline-face)
      (overlay-put highline-overlay 'priority 0))
    ;; move highlight to the current line
    (move-overlay highline-overlay
		  (line-beginning-position)
		  (if highline-whole-line
		      (line-end-position)
		    (1+ (line-end-position))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(add-to-list 'minor-mode-alist '(highline-mode " HL"))
(add-to-list 'minor-mode-alist '(highline-local-mode " hl"))


(provide 'highline)


(run-hooks 'highline-load-hook)


;;; highline.el ends here
